@@ -15,6 +15,7 @@
 import voluptuous as v
 
 from grafana_dashboards.schema.row import Row
+from grafana_dashboards.schema.template import Template
 
 
 class Dashboard(object):
@@ -27,4 +28,7 @@ class Dashboard(object):
         }
         rows = Row().get_schema()
         dashboard.update(rows.schema)
+        templating = Template().get_schema()
+        dashboard.update(templating.schema)
+
         return dashboard
